@@ -37,7 +37,7 @@ static gps_mask_t decode_itk_subframe(struct gps_device_t *, unsigned char *,
 static gps_mask_t decode_itk_navfix(struct gps_device_t *session,
 				    unsigned char *buf, size_t len)
 {
-    unsigned short flags, cflags, pflags;
+    unsigned short flags, pflags;
     gps_mask_t mask = 0;
     double epx, epy, epz, evx, evy, evz, eph;
 
@@ -48,12 +48,12 @@ static gps_mask_t decode_itk_navfix(struct gps_device_t *session,
     }
 
     flags = (ushort) getleu16(buf, 7 + 4);
-    cflags = (ushort) getleu16(buf, 7 + 6);
+    //cflags = (ushort) getleu16(buf, 7 + 6);
     pflags = (ushort) getleu16(buf, 7 + 8);
 
     session->gpsdata.status = STATUS_NO_FIX;
     session->newdata.mode = MODE_NO_FIX;
-    mask = ONLINE_IS | MODE_IS | STATUS_IS | CLEAR_IS;
+    mask = ONLINE_SET | MODE_SET | STATUS_SET | CLEAR_IS;
 
     /* just bail out if this fix is not marked valid */
     if (0 != (pflags & FIX_FLAG_MASK_INVALID)
@@ -63,7 +63,7 @@ static gps_mask_t decode_itk_navfix(struct gps_device_t *session,
     session->newdata.time = gpsd_gpstime_resolve(session,
 	(unsigned short) getles16(buf, 7 + 82),
 	(unsigned int)getleu32(buf, 7 + 84) / 1000.0);
-    mask |= TIME_IS;
+    mask |= TIME_SET | PPSTIME_IS;
 
     epx = (double)(getles32(buf, 7 + 96) / 100.0);
     epy = (double)(getles32(buf, 7 + 100) / 100.0);
@@ -73,7 +73,7 @@ static gps_mask_t decode_itk_navfix(struct gps_device_t *session,
     evz = (double)(getles32(buf, 7 + 194) / 1000.0);
     ecef_to_wgs84fix(&session->newdata, &session->gpsdata.separation,
 		     epx, epy, epz, evx, evy, evz);
-    mask |= LATLON_IS | ALTITUDE_IS | SPEED_IS | TRACK_IS | CLIMB_IS;
+    mask |= LATLON_SET | ALTITUDE_SET | SPEED_SET | TRACK_SET | CLIMB_SET;
     eph = (double)(getles32(buf, 7 + 252) / 100.0);
     /* eph is a circular error, sqrt(epx**2 + epy**2) */
     session->newdata.epx = session->newdata.epy = eph / sqrt(2);
@@ -90,7 +90,7 @@ static gps_mask_t decode_itk_navfix(struct gps_device_t *session,
 	session->gpsdata.dop.pdop = (double)(getleu16(buf, 7 + 60) / 100.0);
 	session->gpsdata.dop.vdop = (double)(getleu16(buf, 7 + 62) / 100.0);
 	session->gpsdata.dop.tdop = (double)(getleu16(buf, 7 + 64) / 100.0);
-	mask |= DOP_IS;
+	mask |= DOP_SET;
     }
 
     if ((pflags & FIX_FLAG_MASK_INVALID) == 0
@@ -107,15 +107,14 @@ static gps_mask_t decode_itk_navfix(struct gps_device_t *session,
     }
 
     gpsd_report(LOG_DATA,
-		"NAV_FIX: time=%.2f, lat=%.2f lon=%.2f alt=%.f speed=%.2f track=%.2f climb=%.2f mode=%d status=%d gdop=%.2f pdop=%.2f hdop=%.2f vdop=%.2f tdop=%.2f mask=%s\n",
+		"NAV_FIX: time=%.2f, lat=%.2f lon=%.2f alt=%.f speed=%.2f track=%.2f climb=%.2f mode=%d status=%d gdop=%.2f pdop=%.2f hdop=%.2f vdop=%.2f tdop=%.2f\n",
 		session->newdata.time, session->newdata.latitude,
 		session->newdata.longitude, session->newdata.altitude,
 		session->newdata.speed, session->newdata.track,
 		session->newdata.climb, session->newdata.mode,
 		session->gpsdata.status, session->gpsdata.dop.gdop,
 		session->gpsdata.dop.pdop, session->gpsdata.dop.hdop,
-		session->gpsdata.dop.vdop, session->gpsdata.dop.tdop,
-		gpsd_maskdump(mask));
+		session->gpsdata.dop.vdop, session->gpsdata.dop.tdop);
     return mask;
 }
 
@@ -154,7 +153,7 @@ static gps_mask_t decode_itk_prnstatus(struct gps_device_t *session,
 	}
 	session->gpsdata.satellites_visible = (int)st;
 	session->gpsdata.satellites_used = (int)nsv;
-	mask = USED_IS | SATELLITE_IS;;
+	mask = USED_IS | SATELLITE_SET;;
 
 	gpsd_report(LOG_DATA,
 		    "PRN_STATUS: time=%.2f visible=%d used=%d mask={USED|SATELLITE}\n",
@@ -193,7 +192,7 @@ static gps_mask_t decode_itk_utcionomodel(struct gps_device_t *session,
     gpsd_report(LOG_DATA,
 		"UTC_IONO_MODEL: time=%.2f mask={TIME}\n",
 		session->newdata.time);
-    return TIME_IS;
+    return TIME_SET | PPSTIME_IS;
 }
 
 static gps_mask_t decode_itk_subframe(struct gps_device_t *session,
@@ -284,8 +283,7 @@ static gps_mask_t italk_parse(struct gps_device_t *session,
 
     type = (uint) getub(buf, 4);
     /* we may need to dump the raw packet */
-    gpsd_report(LOG_RAW, "raw italk packet type 0x%02x length %zu: %s\n",
-		type, len, gpsd_hexdump_wrapper(buf, len, LOG_RAW));
+    gpsd_report(LOG_RAW, "raw italk packet type 0x%02x\n", type);
 
     session->cycle_end_reliable = true;
 
@@ -368,7 +366,7 @@ static gps_mask_t italk_parse(struct gps_device_t *session,
     (void)snprintf(session->gpsdata.tag, sizeof(session->gpsdata.tag),
 		       "ITK-%02x", type);
 
-    return mask | ONLINE_IS;
+    return mask | ONLINE_SET;
 }
 
 /*@ -charint @*/
@@ -414,15 +412,15 @@ const struct gps_type_t italk_binary =
     .parse_packet   = italk_parse_input,/* parse message packets */
     .rtcm_writer    = gpsd_write,	/* send RTCM data straight */
     .event_hook     = NULL,		/* lifetime event handler */
-#ifdef ALLOW_RECONFIGURE
+#ifdef RECONFIGURE_ENABLE
     .speed_switcher = NULL,		/* no speed switcher */
     .mode_switcher  = NULL,		/* no mode switcher */
     .rate_switcher  = NULL,		/* no sample-rate switcher */
     .min_cycle      = 1,		/* not relevant, no rate switch */
-#endif /* ALLOW_RECONFIGURE */
-#ifdef ALLOW_CONTROLSEND
+#endif /* RECONFIGURE_ENABLE */
+#ifdef CONTROLSEND_ENABLE
     .control_send   = NULL,		/* no control string sender */
-#endif /* ALLOW_CONTROLSEND */
+#endif /* CONTROLSEND_ENABLE */
 #ifdef NTPSHM_ENABLE
     .ntp_offset     = NULL,		/* no method for NTP fudge factor */
 #endif /* NTPSHM_ ENABLE */
