@@ -22,20 +22,23 @@ passing RTCM corrections to a GPS, which normally should just be
 passed an entire correction packet for processing by their internal
 firmware.
 
+Decodes of the following types have been verified: 1004, 1005, 1006,
+1008, 1012, 1013, 1029. There is good reason to believe the 1007 code
+is correct, as it's identical to 1008 up to where it ends.
+
+The 1033 decode was arrived at by looking at an rtcminspect dump and noting
+that it carries an information superset of the 1008.  There are additional
+Receiver and Firmware fields we're not certain to decode without access
+to an RTCM3 standard at revision 4 or later, but the guess in the code
+has been observed to correctly analyze a message with a nonempty Receiver
+field.
+
 This file is Copyright (c) 2010 by the GPSD project
 BSD terms apply: see the file COPYING in the distribution root for details.
 
 *****************************************************************************/
 
-#include <stdio.h>
 #include <string.h>
-#include <assert.h>
-#include "gpsd_config.h"
-#ifndef S_SPLINT_S
-#include <sys/types.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>		/* for ntohl(3) and friends */
-#endif /* S_SPLINT_S */
 
 #include "gpsd.h"
 #include "bits.h"
@@ -43,17 +46,20 @@ BSD terms apply: see the file COPYING in the distribution root for details.
 #ifdef RTCM104V3_ENABLE
 
 /* scaling constants for RTCM3 real number types */
-#define GPS_PSEUDORANGE_RESOLUTION	0.2	/* DF011 */
+#define GPS_PSEUDORANGE_RESOLUTION	0.02	/* DF011 */
 #define PSEUDORANGE_DIFF_RESOLUTION	0.0005	/* DF012,DF042 */
-#define CARRIER_NOISE_RATIO_UNITS	0.25	/* DF015 */
+#define CARRIER_NOISE_RATIO_UNITS	0.25	/* DF015, DF045, DF50 */
 #define ANTENNA_POSITION_RESOLUTION	0.0001	/* DF025-027 */
-#define GLONASS_PSEUDORANGE_RESOLUTION	0.2	/* DF041 */
+#define GLONASS_PSEUDORANGE_RESOLUTION	0.02	/* DF041 */
 #define ANTENNA_DEGREE_RESOLUTION	25e-6	/* DF062 */
 #define GPS_EPOCH_TIME_RESOLUTION	0.1	/* DF065 */
 #define PHASE_CORRECTION_RESOLUTION	0.5	/* DF069-070 */
 
+
 /* Other magic values */
-#define GPS_INVALID_PSEUDORANGE		0x80000	/* DF012 */
+#define GPS_INVALID_PSEUDORANGE		0x80000	/* DF012, DF018 */
+#define GLONASS_INVALID_RANGEINCR	0x2000	/* DF047 */
+#define GLONASS_CHANNEL_BASE		7	/* DF040 */
 
 /* Large case statements make GNU indent very confused */
 /* *INDENT-OFF* */
@@ -62,7 +68,7 @@ BSD terms apply: see the file COPYING in the distribution root for details.
 void rtcm3_unpack( /*@out@*/ struct rtcm3_t *rtcm, char *buf)
 /* break out the raw bits into the scaled report-structure fields */
 {
-    unsigned int n, n2;
+    unsigned int n, n2, n3, n4;
     int bitcount = 0;
     unsigned int i;
     signed long temp;
@@ -90,9 +96,8 @@ void rtcm3_unpack( /*@out@*/ struct rtcm3_t *rtcm, char *buf)
     rtcm->length = (uint)ugrab(10);
     rtcm->type = (uint)ugrab(12);
 
-    gpsd_report(LOG_RAW, "RTCM3: type %d payload length %d: %s\n",
-		rtcm->type, rtcm->length, 
-		gpsd_hexdump_wrapper(buf+3, rtcm->length, LOG_RAW));
+    gpsd_report(LOG_RAW, "RTCM3: type %d payload length %d\n",
+		rtcm->type, rtcm->length);
 
     switch (rtcm->type) {
     case 1001:			/* GPS Basic RTK, L1 Only */
@@ -111,8 +116,6 @@ void rtcm3_unpack( /*@out@*/ struct rtcm3_t *rtcm, char *buf)
 	    R1001.L1.locktime =	(unsigned char)sgrab(7);
 	}
 #undef R1001
-	assert(bitcount ==
-	       64 + 58 * rtcm->rtcmtypes.rtcm3_1001.header.satcount);
 	break;
 
     case 1002:			/* GPS Extended RTK, L1 Only */
@@ -133,8 +136,6 @@ void rtcm3_unpack( /*@out@*/ struct rtcm3_t *rtcm, char *buf)
 	    R1002.L1.CNR = (ugrab(8)) * CARRIER_NOISE_RATIO_UNITS;
 	}
 #undef R1002
-	assert(bitcount ==
-	       64 + 74 * rtcm->rtcmtypes.rtcm3_1002.header.satcount);
 	break;
 
     case 1003:			/* GPS Basic RTK, L1 & L2 */
@@ -162,8 +163,6 @@ void rtcm3_unpack( /*@out@*/ struct rtcm3_t *rtcm, char *buf)
 	    R1003.L2.locktime =	(unsigned char)sgrab(7);
 	}
 #undef R1003
-	assert(bitcount ==
-	       64 + 101 * rtcm->rtcmtypes.rtcm3_1003.header.satcount);
 	break;
 
     case 1004:			/* GPS Extended RTK, L1 & L2 */
@@ -183,28 +182,19 @@ void rtcm3_unpack( /*@out@*/ struct rtcm3_t *rtcm, char *buf)
 	    R1004.L1.ambiguity = (unsigned char)ugrab(8);
 	    R1004.L1.CNR = ugrab(8) * CARRIER_NOISE_RATIO_UNITS;
 	    R1004.L2.indicator = (unsigned char)ugrab(2);
-	    GPS_PSEUDORANGE(R1004.L2, 24);
+	    GPS_PSEUDORANGE(R1004.L2, 14);
 	    RANGEDIFF(R1004.L2, 20);
 	    R1004.L2.locktime =	(unsigned char)sgrab(7);
-	    R1004.L2.ambiguity = (unsigned char)ugrab(8);
 	    R1004.L2.CNR = ugrab(8) * CARRIER_NOISE_RATIO_UNITS;
 	}
 #undef R1004
-	assert(bitcount ==
-	       64 + 125 * rtcm->rtcmtypes.rtcm3_1004.header.satcount);
 	break;
 
     case 1005:			/* Stationary Antenna Reference Point, No Height Information */
 #define R1005 rtcm->rtcmtypes.rtcm3_1005
 	R1005.station_id = (unsigned short)ugrab(12);
 	ugrab(6);		/* reserved */
-	temp = ugrab(3);
-	if ((temp & 0x04)!=0)
-	    R1005.system = NAVSYSTEM_GPS;
-	if ((temp & 0x02)!=0)
-	    R1005.system = NAVSYSTEM_GLONASS;
-	if ((temp & 0x01)!=0)
-	    R1005.system = NAVSYSTEM_GALILEO;
+	R1005.system = ugrab(3);
 	R1005.reference_station = (bool)ugrab(1);
 	R1005.ecef_x = sgrab(38) * ANTENNA_POSITION_RESOLUTION;
 	R1005.single_receiver = ugrab(1);
@@ -218,14 +208,8 @@ void rtcm3_unpack( /*@out@*/ struct rtcm3_t *rtcm, char *buf)
     case 1006:			/* Stationary Antenna Reference Point, with Height Information */
 #define R1006 rtcm->rtcmtypes.rtcm3_1006
 	R1006.station_id = (unsigned short)ugrab(12);
-	ugrab(6);		/* reserved */
-	temp = ugrab(3);
-	if ((temp & 0x04)!=0)
-	    R1006.system = NAVSYSTEM_GPS;
-	if ((temp & 0x02)!=0)
-	    R1006.system = NAVSYSTEM_GLONASS;
-	if ((temp & 0x01)!=0)
-	    R1006.system = NAVSYSTEM_GALILEO;
+	(void)ugrab(6);		/* reserved */
+	R1006.system = ugrab(3);
 	R1006.reference_station = (bool)ugrab(1);
 	R1006.ecef_x = sgrab(38) * ANTENNA_POSITION_RESOLUTION;
 	R1006.single_receiver = ugrab(1);
@@ -235,31 +219,28 @@ void rtcm3_unpack( /*@out@*/ struct rtcm3_t *rtcm, char *buf)
 	R1006.ecef_z = sgrab(38) * ANTENNA_POSITION_RESOLUTION;
 	R1006.height = ugrab(16) * ANTENNA_POSITION_RESOLUTION;
 #undef R1006
-	assert(bitcount == 168);
 	break;
 
     case 1007:			/* Antenna Descriptor */
 	rtcm->rtcmtypes.rtcm3_1007.station_id = (unsigned short)ugrab(12);
 	n = (unsigned long)ugrab(8);
-	(void)memcpy(rtcm->rtcmtypes.rtcm3_1007.descriptor, buf + 4, n);
+	(void)memcpy(rtcm->rtcmtypes.rtcm3_1007.descriptor, buf + 7, n);
 	rtcm->rtcmtypes.rtcm3_1007.descriptor[n] = '\0';
 	bitcount += 8 * n;
 	rtcm->rtcmtypes.rtcm3_1007.setup_id = ugrab(8);
-	assert(bitcount == (int)(40 + 8 * n));
 	break;
 
     case 1008:			/* Antenna Descriptor & Serial Number */
 	rtcm->rtcmtypes.rtcm3_1008.station_id = (unsigned short)ugrab(12);
 	n = (unsigned long)ugrab(8);
-	(void)memcpy(rtcm->rtcmtypes.rtcm3_1008.descriptor, buf + 4, n);
+	(void)memcpy(rtcm->rtcmtypes.rtcm3_1008.descriptor, buf + 7, n);
 	rtcm->rtcmtypes.rtcm3_1008.descriptor[n] = '\0';
 	bitcount += 8 * n;
 	rtcm->rtcmtypes.rtcm3_1008.setup_id = ugrab(8);
 	n2 = (unsigned long)ugrab(8);
-	(void)memcpy(rtcm->rtcmtypes.rtcm3_1008.serial, buf + 6 + n, n2);
+	(void)memcpy(rtcm->rtcmtypes.rtcm3_1008.serial, buf + 9 + n, n2);
 	rtcm->rtcmtypes.rtcm3_1008.serial[n2] = '\0';
 	bitcount += 8 * n2;
-	assert(bitcount == (int)(48 + 8 * (n + n2)));
 	break;
 
     case 1009:			/* GLONASS Basic RTK, L1 Only */
@@ -274,14 +255,12 @@ void rtcm3_unpack( /*@out@*/ struct rtcm3_t *rtcm, char *buf)
 	for (i = 0; i < rtcm->rtcmtypes.rtcm3_1009.header.satcount; i++) {
 	    R1009.ident = (ushort)ugrab(6);
 	    R1009.L1.indicator = (bool)ugrab(1);
-	    R1009.L1.channel = (ushort)ugrab(5);
+	    R1009.L1.channel = (short)ugrab(5) - GLONASS_CHANNEL_BASE;
 	    R1009.L1.pseudorange = ugrab(25) * GLONASS_PSEUDORANGE_RESOLUTION;
 	    RANGEDIFF(R1009.L1, 20);
 	    R1009.L1.locktime =	(unsigned char)sgrab(7);
 	}
 #undef R1009
-	assert(bitcount ==
-	       61 + 64 * rtcm->rtcmtypes.rtcm3_1009.header.satcount);
 	break;
 
     case 1010:			/* GLONASS Extended RTK, L1 Only */
@@ -296,7 +275,7 @@ void rtcm3_unpack( /*@out@*/ struct rtcm3_t *rtcm, char *buf)
 	for (i = 0; i < rtcm->rtcmtypes.rtcm3_1010.header.satcount; i++) {
 	    R1010.ident = (ushort)ugrab(6);
 	    R1010.L1.indicator = (bool)ugrab(1);
-	    R1010.L1.channel = (ushort)ugrab(5);
+	    R1010.L1.channel = (short)ugrab(5) - GLONASS_CHANNEL_BASE;
 	    R1010.L1.pseudorange = ugrab(25) * GLONASS_PSEUDORANGE_RESOLUTION;
 	    RANGEDIFF(R1010.L1, 20);
 	    R1010.L1.locktime =	(unsigned char)sgrab(7);
@@ -304,8 +283,6 @@ void rtcm3_unpack( /*@out@*/ struct rtcm3_t *rtcm, char *buf)
 	    R1010.L1.CNR = ugrab(8) * CARRIER_NOISE_RATIO_UNITS;
 	}
 #undef R1010
-	assert(bitcount ==
-	       61 + 79 * rtcm->rtcmtypes.rtcm3_1010.header.satcount);
 	break;
 
     case 1011:			/* GLONASS Basic RTK, L1 & L2 */
@@ -320,14 +297,14 @@ void rtcm3_unpack( /*@out@*/ struct rtcm3_t *rtcm, char *buf)
 	for (i = 0; i < rtcm->rtcmtypes.rtcm3_1011.header.satcount; i++) {
 	    R1011.ident = (ushort)ugrab(6);
 	    R1011.L1.indicator = (bool)ugrab(1);
-	    R1011.L1.channel = (ushort)ugrab(5);
+	    R1011.L1.channel = (short)ugrab(5) - GLONASS_CHANNEL_BASE;
 	    R1011.L1.pseudorange = ugrab(25) * GLONASS_PSEUDORANGE_RESOLUTION;
 	    RANGEDIFF(R1011.L1, 20);
 	    R1011.L1.locktime =	(unsigned char)sgrab(7);
 	    R1011.L1.ambiguity = (unsigned char)ugrab(7);
 	    R1011.L1.CNR = ugrab(8) * CARRIER_NOISE_RATIO_UNITS;
 	    R1011.L2.indicator = (bool)ugrab(1);
-	    R1011.L2.channel = (ushort)ugrab(5);
+	    R1011.L2.channel = (short)ugrab(5) - GLONASS_CHANNEL_BASE;
 	    R1011.L2.pseudorange = ugrab(25) * GLONASS_PSEUDORANGE_RESOLUTION;
 	    RANGEDIFF(R1011.L2, 20);
 	    R1011.L2.locktime =	(unsigned char)sgrab(7);
@@ -335,8 +312,6 @@ void rtcm3_unpack( /*@out@*/ struct rtcm3_t *rtcm, char *buf)
 	    R1011.L2.CNR = ugrab(8) * CARRIER_NOISE_RATIO_UNITS;
 	}
 #undef R1011
-	assert(bitcount ==
-	       61 + 107 * rtcm->rtcmtypes.rtcm3_1011.header.satcount);
 	break;
 
     case 1012:			/* GLONASS Extended RTK, L1 & L2 */
@@ -349,20 +324,26 @@ void rtcm3_unpack( /*@out@*/ struct rtcm3_t *rtcm, char *buf)
 	rtcm->rtcmtypes.rtcm3_1012.header.interval = (ushort)ugrab(3);
 #define R1012 rtcm->rtcmtypes.rtcm3_1012.rtk_data[i]
 	for (i = 0; i < rtcm->rtcmtypes.rtcm3_1012.header.satcount; i++) {
+	    unsigned int rangeincr;
 	    R1012.ident = (ushort)ugrab(6);
 	    R1012.L1.indicator = (bool)ugrab(1);
-	    R1012.L1.channel = (ushort)ugrab(5);
+	    R1012.L1.channel = (short)ugrab(5) - GLONASS_CHANNEL_BASE;
 	    R1012.L1.pseudorange = ugrab(25) * GLONASS_PSEUDORANGE_RESOLUTION;
 	    RANGEDIFF(R1012.L1, 20);
-	    R1012.L1.locktime =	(unsigned char)sgrab(7);
-	    R1012.L2.indicator = (bool)ugrab(1);
-	    R1012.L2.pseudorange = ugrab(25) * GLONASS_PSEUDORANGE_RESOLUTION;
+	    R1012.L1.locktime =	(unsigned char)ugrab(7);
+	    R1012.L1.ambiguity = (unsigned char)ugrab(7);
+	    R1012.L1.CNR = (unsigned char)ugrab(8) * CARRIER_NOISE_RATIO_UNITS;
+	    R1012.L2.indicator = (bool)ugrab(2);
+	    rangeincr = ugrab(14);
+	    if (rangeincr == GLONASS_INVALID_RANGEINCR)
+		R1012.L2.pseudorange = 0;
+	    else
+		R1012.L2.pseudorange = (rangeincr * GLONASS_PSEUDORANGE_RESOLUTION);
 	    RANGEDIFF(R1012.L2, 20);
 	    R1012.L2.locktime =	(unsigned char)sgrab(7);
+	    R1012.L2.CNR = (unsigned char)ugrab(8) * CARRIER_NOISE_RATIO_UNITS;
 	}
 #undef R1012
-	assert(bitcount ==
-	       61 + 130 * rtcm->rtcmtypes.rtcm3_1012.header.satcount);
 	break;
 
     case 1013:			/* System Parameters */
@@ -378,7 +359,6 @@ void rtcm3_unpack( /*@out@*/ struct rtcm3_t *rtcm, char *buf)
 	    R1013.interval = (unsigned short)ugrab(16);
 	}
 #undef R1013
-	assert(bitcount == 70 + 29 * rtcm->rtcmtypes.rtcm3_1013.ncount);
 	break;
 
     case 1014:
@@ -392,25 +372,6 @@ void rtcm3_unpack( /*@out@*/ struct rtcm3_t *rtcm, char *buf)
 	rtcm->rtcmtypes.rtcm3_1014.d_lon =
 	    (unsigned short)ugrab(21) * ANTENNA_DEGREE_RESOLUTION;
 	rtcm->rtcmtypes.rtcm3_1014.d_alt = (unsigned short)ugrab(23) / 1000;
-	assert(bitcount == 117);
-	break;
-
-    case 1015:
-	break;
-
-    case 1016:
-	break;
-
-    case 1017:
-	break;
-
-    case 1018:
-	break;
-
-    case 1019:
-	break;
-
-    case 1020:
 	break;
 
     case 1029:
@@ -418,11 +379,32 @@ void rtcm3_unpack( /*@out@*/ struct rtcm3_t *rtcm, char *buf)
 	rtcm->rtcmtypes.rtcm3_1029.mjd = (unsigned short)ugrab(16);
 	rtcm->rtcmtypes.rtcm3_1029.sod = (unsigned short)ugrab(17);
 	rtcm->rtcmtypes.rtcm3_1029.len = (unsigned long)ugrab(7);
-	n = rtcm->rtcmtypes.rtcm3_1029.unicode_units = (size_t)ugrab(8);
+	rtcm->rtcmtypes.rtcm3_1029.unicode_units = (size_t)ugrab(8);
 	(void)memcpy(rtcm->rtcmtypes.rtcm3_1029.text, 
 		     buf + 12, rtcm->rtcmtypes.rtcm3_1029.unicode_units);
 	break;
 
+    case 1033:			/* see note in header */
+	rtcm->rtcmtypes.rtcm3_1033.station_id = (unsigned short)ugrab(12);
+	n = (unsigned long)ugrab(8);
+	(void)memcpy(rtcm->rtcmtypes.rtcm3_1033.descriptor, buf + 7, n);
+	rtcm->rtcmtypes.rtcm3_1033.descriptor[n] = '\0';
+	bitcount += 8 * n;
+	rtcm->rtcmtypes.rtcm3_1033.setup_id = ugrab(8);
+	n2 = (unsigned long)ugrab(8);
+	(void)memcpy(rtcm->rtcmtypes.rtcm3_1033.serial, buf + 9 + n, n2);
+	rtcm->rtcmtypes.rtcm3_1033.serial[n2] = '\0';
+	bitcount += 8 * n2;
+	n3 = (unsigned long)ugrab(8);
+	(void)memcpy(rtcm->rtcmtypes.rtcm3_1033.receiver, buf + 10+n+n2, n3);
+	rtcm->rtcmtypes.rtcm3_1033.receiver[n3] = '\0';
+	bitcount += 8 * n3;
+	n4 = (unsigned long)ugrab(8);
+	(void)memcpy(rtcm->rtcmtypes.rtcm3_1033.firmware, buf + 11+n+n2+n3, n3);
+	rtcm->rtcmtypes.rtcm3_1033.firmware[n4] = '\0';
+	bitcount += 8 * n4;
+	break;
+
     default:
 	/* 
 	 * Leader bytes, message length, and checksum won't be copied.
