@@ -2,7 +2,8 @@
  * This is the gpsd driver for SiRF GPSes operating in binary mode.
  * It also handles uBlox, a SiRF derivative.
  *
- * The advantage: Reports climb/sink rate (raw-mode clients won't see this).
+ * The advantages: Reports climb/sink rate (raw-mode clients won't see this).
+ * Also, we can flag DGPS satellites used in the skyview when SBAS is in use. 
  * The disadvantages: Doesn't return PDOP or VDOP, just HDOP.
  *
  * Chris Kuethe, our SiRF expert, tells us:
@@ -137,6 +138,8 @@ static gps_mask_t sirf_msg_swversion(struct gps_device_t *, unsigned char *,
 				     size_t);
 static gps_mask_t sirf_msg_sysparam(struct gps_device_t *, unsigned char *,
 				    size_t);
+static gps_mask_t sirf_msg_dgpsstatus(struct gps_device_t *, unsigned char *,
+				      size_t);
 static gps_mask_t sirf_msg_ublox(struct gps_device_t *, unsigned char *,
 				 size_t);
 
@@ -225,8 +228,10 @@ static bool sirfbin_speed(struct gps_device_t *session, speed_t speed, char pari
 	parity = (char)0;
 	break;
     }
-    msg[7] = (unsigned char)HI(speed);
-    msg[8] = (unsigned char)LO(speed);
+    msg[5] = (unsigned char)((speed >> 24) & 0xff);
+    msg[6] = (unsigned char)((speed >> 16) & 0xff);
+    msg[7] = (unsigned char)((speed >> 8) & 0xff);
+    msg[8] = (unsigned char)(speed & 0xff);
     msg[10] = (unsigned char)stopbits;
     msg[11] = (unsigned char)parity;
     return (sirf_write(session, msg));
@@ -254,6 +259,12 @@ static bool sirf_to_nmea(struct gps_device_t *session, speed_t speed)
     };
     /*@ -charint @*/
 
+    if (speed >= 0xffff) {
+	gpsd_report(LOG_ERROR, "SiRF: can't switch from SiRF to NMEA because current speed %u is big.",
+		     speed);
+	return false;
+    }
+
     msg[26] = (unsigned char)HI(speed);
     msg[27] = (unsigned char)LO(speed);
     return (sirf_write(session, msg));
@@ -469,7 +480,6 @@ static gps_mask_t sirf_msg_svinfo(struct gps_device_t *session,
 				  unsigned char *buf, size_t len)
 {
     int st, i, j, cn;
-    gps_mask_t mask = 0;
 
     if (len != 188)
 	return 0;
@@ -507,6 +517,14 @@ static gps_mask_t sirf_msg_svinfo(struct gps_device_t *session,
 	    st += 1;
     }
     session->gpsdata.satellites_visible = st;
+    /* mark SBAS sats in use if SBAS was in use as of the last MID 27 */
+    for (i = 0; i < st; i++) {
+	int prn = session->gpsdata.PRN[i];
+	if (DGPS_PRN(prn) \
+		&& session->gpsdata.status == STATUS_DGPS_FIX \
+		&& session->driver.sirf.dgps_source == SIRF_DGPS_SOURCE_SBAS)
+	    session->gpsdata.used[session->gpsdata.satellites_used++] = prn;
+    }
 #ifdef NTPSHM_ENABLE
     if (st < 3) {
 	gpsd_report(LOG_PROG,
@@ -518,22 +536,11 @@ static gps_mask_t sirf_msg_svinfo(struct gps_device_t *session,
 		    session->driver.sirf.time_seen,
 		    session->gpsdata.skyview_time,
 		    session->context->leap_seconds);
-	/*
-	 * Might be better if we didn't change the fix timestamp here,
-	 * but presently there's no other way to pass the time to NTP.
-	 */
-	session->newdata.time = session->gpsdata.skyview_time;
-	mask |= TIME_SET | PPSTIME_IS;
-	/*
-	 * This time stamp, at 4800bps, is so close to 1 sec old as to
-	 * be confusing to ntpd, but ntpshm_put() will ignore it if a better
-	 * time already seen
-	 */
     }
 #endif /* NTPSHM_ENABLE */
     gpsd_report(LOG_DATA, "SiRF: MTD 0x04: visible=%d mask={SATELLITE}\n",
 		session->gpsdata.satellites_visible);
-    return SATELLITE_SET | mask;
+    return SATELLITE_SET;
 }
 
 #ifdef NTPSHM_ENABLE
@@ -851,6 +858,14 @@ static gps_mask_t sirf_msg_sysparam(struct gps_device_t *session,
     return 0;
 }
 
+static gps_mask_t sirf_msg_dgpsstatus(struct gps_device_t *session,
+				 unsigned char *buf, size_t len UNUSED)
+/* only documentented from prorocol version 1.7 (2005) onwards */
+{
+    session->driver.sirf.dgps_source = (unsigned int)getub(buf, 1);
+    return 0;
+}
+
 static gps_mask_t sirf_msg_ublox(struct gps_device_t *session,
 				 unsigned char *buf, size_t len UNUSED)
 {
@@ -964,8 +979,9 @@ static gps_mask_t sirf_msg_ppstime(struct gps_device_t *session,
 	    gpsd_report(LOG_RAW, "SiRF: NTPD just SEEN_UTC_2\n");
 	}
 	gpsd_report(LOG_PROG,
-		    "SiRF: NTPD valid time MID 0x34, seen=0x%02x\n",
-		    session->driver.sirf.time_seen);
+		    "SiRF: NTPD valid time MID 0x34, seen=0x%02x, leap=%d\n",
+		    session->driver.sirf.time_seen,
+		    session->context->leap_seconds);
 	session->driver.sirf.time_seen |= TIME_SEEN_UTC_2;
 #endif /* NTPSHM_ENABLE */
 	mask |= TIME_SET;
@@ -1073,9 +1089,8 @@ gps_mask_t sirf_parse(struct gps_device_t * session, unsigned char *buf,
     case 0x13:			/* Navigation Parameters MID 19 */
 	return sirf_msg_sysparam(session, buf, len);
 
-    case 0x1b:			/* DGPS status (undocumented) MID 27 */
-	gpsd_report(LOG_PROG, "SiRF: unused DGPSF 0x1b\n");
-	return 0;
+    case 0x1b:			/* DGPS status MID 27 */
+	return sirf_msg_dgpsstatus(session, buf, len);
 
     case 0x1c:			/* Navigation Library Measurement Data MID 28 */
 	gpsd_report(LOG_PROG, "SiRF: NLMD 0x1c\n");
@@ -1093,7 +1108,7 @@ gps_mask_t sirf_parse(struct gps_device_t * session, unsigned char *buf,
 	gpsd_report(LOG_PROG, "SiRF: unused NLID 0x1f\n");
 	return 0;
 
-    case 0x29:			/* Geodetic Navigation Information MID 41 */
+    case 0x29:			/* Geodetic Navigation Data MID 41 */
 	gpsd_report(LOG_PROG, "SiRF: unused GND 0x29\n");
 	return 0;
 
