@@ -61,11 +61,6 @@
 #define PROTO_TTY "/dev/ttyS0"	/* correct for Linux */
 #endif
 
-/* Name of (unprivileged) user to change to when we drop privileges. */
-#ifndef GPSD_USER
-#define GPSD_USER	"nobody"
-#endif
-
 /*
  * Timeout policy.  We can't rely on clients closing connections
  * correctly, so we need timeouts to tell us when it's OK to
@@ -138,7 +133,9 @@ static const int af = AF_INET;
 static fd_set all_fds;
 static int maxfd;
 static bool in_background = false;
+#ifndef FORCE_GLOBAL_ENABLE
 static bool listen_global = false;
+#endif /* FORCE_GLOBAL_ENABLE */
 #ifndef FORCE_NOWAIT
 static bool nowait = false;
 #endif /* FORCE_NOWAIT */
@@ -251,9 +248,11 @@ static void usage(void)
   -b		     	    = bluetooth-safe: open data sources read-only\n\
   -n			    = don't wait for client connects to poll GPS\n\
   -N			    = don't go into background\n\
-  -F sockfile		    = specify control socket location\n\
-  -G         		    = make gpsd listen on INADDR_ANY\n\
-  -P pidfile	      	    = set file to record process ID \n\
+  -F sockfile		    = specify control socket location\n"
+#ifndef FORCE_GLOBAL_ENABLE
+"  -G         		    = make gpsd listen on INADDR_ANY\n"
+#endif /* FORCE_GLOBAL_ENABLE */
+"  -P pidfile	      	    = set file to record process ID \n\
   -D integer (default 0)    = set debug level \n\
   -S integer (default %s) = set port for daemon \n\
   -h		     	    = help message \n\
@@ -383,10 +382,12 @@ static int passivesock_af(int af, char *service, char *tcp_or_udp, int qlen)
 
 	memset((char *)&sat.sa_in, 0, sin_len);
 	sat.sa_in.sin_family = (sa_family_t) AF_INET;
-	if (listen_global)
-	    sat.sa_in.sin_addr.s_addr = htonl(INADDR_ANY);
-	else
+#ifndef FORCE_GLOBAL_ENABLE
+	if (!listen_global)
 	    sat.sa_in.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
+	else
+#endif /* FORCE_GLOBAL_ENABLE */
+	    sat.sa_in.sin_addr.s_addr = htonl(INADDR_ANY);
 	sat.sa_in.sin_port = htons(port);
 
 	af_str = "IPv4";
@@ -399,17 +400,12 @@ static int passivesock_af(int af, char *service, char *tcp_or_udp, int qlen)
 
 	memset((char *)&sat.sa_in6, 0, sin_len);
 	sat.sa_in6.sin6_family = (sa_family_t) AF_INET6;
-	if (listen_global) {
-	    /* BAD:  sat.sa_in6.sin6_addr = in6addr_any;
-	     * the simple assignment will not work (except as an initializer)
-	     * because sin6_addr is an array not a simple type
-	     * we could do something like this:
-	     * memcpy(sat.sa_in6.sin6_addr, in6addr_any, sizeof(sin6_addr));
-	     * BUT, all zeros is IPv6 wildcard, and we just zeroed the array
-	     * so really nothing to do here
-	     */
-	} else
-	    sat.sa_in6.sin6_addr = in6addr_loopback;
+#ifndef FORCE_GLOBAL_ENABLE
+	if (!listen_global)
+	    sat.sa_in6.sin6_addr = in6addr_loopback;	
+	else
+#endif /* FORCE_GLOBAL_ENABLE */
+	    sat.sa_in6.sin6_addr = in6addr_any;
 	sat.sa_in6.sin6_port = htons(port);
 
 	/*
@@ -422,6 +418,17 @@ static int passivesock_af(int af, char *service, char *tcp_or_udp, int qlen)
 	 */
 	af_str = "IPv6";
 	s = socket(PF_INET6, type, proto);
+
+	/*
+	 * On some network stacks, including Linux's, an IPv6 socket
+	 * defaults to listening on IPv4 as well. Unless we disable
+	 * this, trying to listen on in6addr_any will fail with the
+	 * address-in-use error condition.
+	 */
+	{
+	    int on = 1;
+	    (void)setsockopt(s, IPPROTO_IPV6, IPV6_V6ONLY, &on, sizeof(on));
+	}
 	break;
 #endif
     default:
@@ -762,7 +769,6 @@ static void handle_control(int sfd, char *buf)
 	    else
 		ignore_return(write(sfd, "ERROR\n", 6));
 	}
-#ifdef __UNUSED__
     } else if (buf[0] == '!') {
 	/* split line after ! into device=string, send string to device */
 	char *eq;
@@ -850,7 +856,6 @@ static void handle_control(int sfd, char *buf)
 	    ignore_return(write(sfd, "\n", 1));
 	}
 	ignore_return(write(sfd, "OK\n", 6));
-#endif /* __UNUSED__ */
     } else {
 	/* unknown command */
 	ignore_return(write(sfd, "ERROR\n", 6));
@@ -1029,6 +1034,9 @@ static void handle_request(struct subscriber_t *sub,
 	    ++buf;
 	} else {
 	    int status = json_watch_read(buf + 1, &sub->policy, &end);
+#ifndef TIMING_ENABLE
+	    sub->policy.timing = false;
+#endif /* TIMING_ENABLE */
 	    if (end == NULL)
 		buf += strlen(buf);
 	    else {
@@ -1219,7 +1227,7 @@ static void handle_request(struct subscriber_t *sub,
 	for (devp = devices; devp < devices + MAXDEVICES; devp++) {
 	    if (allocated_device(devp) && subscribed(sub, devp)) {
 		if ((devp->observed & GPS_TYPEMASK) != 0) {
-		    json_tpv_dump(&devp->gpsdata,
+		    json_tpv_dump(devp, &sub->policy,
 				  reply + strlen(reply),
 				  replylen - strlen(reply));
 		    rstrip(reply);
@@ -1621,28 +1629,11 @@ static void consume_packets(struct gps_device_t *device)
 			char buf[GPS_JSON_RESPONSE_MAX * 4];
 
 			json_data_report(changed,
-					 &device->gpsdata, &sub->policy,
+					 device, &sub->policy,
 					 buf, sizeof(buf));
 			if (buf[0] != '\0')
 			    (void)throttled_write(sub, buf, strlen(buf));
 
-#ifdef TIMING_ENABLE
-			if (buf[0] != '\0' && sub->policy.timing) {
-			    (void)snprintf(buf, sizeof(buf),
-					   "{\"class\":\"TIMING\","
-					   "\"tag\":\"%s\",\"len\":%d,"
-					   "\"xmit\":%lf,\"recv\":%lf,"
-					   "\"decode\":%lf,"
-					   "\"emit\":%lf}\r\n",
-					   device->gpsdata.tag,
-					   (int)device->packet.outbuflen,
-					   device->d_xmit_time,
-					   device->d_recv_time,
-					   device->d_decode_time,
-					   timestamp());
-			    (void)throttled_write(sub, buf, strlen(buf));
-			}
-#endif /* TIMING_ENABLE */
 		    }
 		}
 	    }
@@ -1804,9 +1795,11 @@ int main(int argc, char *argv[])
 	case 'b':
 	    context.readonly = true;
 	    break;
+#ifndef FORCE_GLOBAL_ENABLE
 	case 'G':
 	    listen_global = true;
 	    break;
+#endif /* FORCE_GLOBAL_ENABLE */
 	case 'l':		/* list known device types and exit */
 	    for (dp = gpsd_drivers; *dp; dp++) {
 #ifdef RECONFIGURE_ENABLE
@@ -2016,6 +2009,13 @@ int main(int argc, char *argv[])
 	 * permissions set.
 	 */
 	/*@-type@*/
+#ifdef GPSD_GROUP
+	{
+	    struct group *grp = getgrnam(GPSD_GROUP);
+	    if (grp)
+		(void)setgid(grp->gr_gid);
+	}
+#else
 	if ((optind < argc && stat(argv[optind], &stb) == 0)
 	    || stat(PROTO_TTY, &stb) == 0) {
 	    gpsd_report(LOG_PROG, "changing to group %d\n", stb.st_gid);
@@ -2023,12 +2023,6 @@ int main(int argc, char *argv[])
 		gpsd_report(LOG_ERROR, "setgid() failed, errno %s\n",
 			    strerror(errno));
 	}
-#ifdef GPSD_GROUP
-	else {
-	    struct group *grp = getgrnam(GPSD_GROUP);
-	    if (grp)
-		(void)setgid(grp->gr_gid);
-	}
 #endif
 	pw = getpwnam(GPSD_USER);
 	if (pw)
