@@ -21,8 +21,7 @@
   A client that passes gpsd data to lcdproc, turning your car computer
   into a very expensive feature-free GPS receiver ;^).  Currently
   assumes a 4x40 LCD and writes data formatted to fit that size
-  screen.  Also displays 4- or 6-character Maidenhead grid square
-  output for the hams among us.
+  screen.  Also displays Maidenhead grid square output for the hams among us.
 
   This program assumes that LCDd (lcdproc) is running locally on the
   default (13666) port.  The #defines LCDDHOST and LCDDPORT can be
@@ -62,7 +61,6 @@
 #include "revision.h"
 
 /* Prototypes. */
-void latlon2maidenhead(char *st,float n,float e);
 ssize_t sockreadline(int sockd,void *vptr,size_t maxlen);
 ssize_t sockwriteline(int sockd,const void *vptr,size_t n);
 int send_lcd(char *buf);
@@ -78,34 +76,6 @@ double avgclimb, climb[CLIMB];
 /* Global socket descriptor for LCDd. */
 int sd;
 
-/* Convert lat/lon to Maidenhead.  Lifted from QGrid -
-   http://users.pandora.be/on4qz/qgrid/ */
-void latlon2maidenhead(char *st,float n,float e)
-{
-  int t1;
-  e=e+180.0;
-  t1=(int)(e/20);
-  st[0]=t1+'A';
-  e-=(float)t1*20.0;
-  t1=(int)e/2;
-  st[2]=t1+'0';
-  e-=(float)t1*2;
-#ifndef CLIMB
-  st[4]=(int)(e*12.0+0.5)+'A';
-#endif
-
-  n=n+90.0;
-  t1=(int)(n/10.0);
-  st[1]=t1+'A';
-  n-=(float)t1*10.0;
-  st[3]=(int)n+'0';
-  n-=(int)n;
-  n*=24; // convert to 24 division
-#ifndef CLIMB
-  st[5]=(int)(n+0.5)+'A';
-#endif
-}
-
 /*  Read a line from a socket  */
 ssize_t sockreadline(int sockd,void *vptr,size_t maxlen) {
   ssize_t n,rc;
@@ -174,7 +144,7 @@ int send_lcd(char *buf) {
   }
 
   /* send the command */
-  res=sockwriteline(sd,buf,outlen);
+  (void)sockwriteline(sd,buf,outlen);
 
   /* TODO:  check return status */
 
@@ -210,14 +180,12 @@ static enum deg_str_type deg_type = deg_dd;
 static void update_lcd(struct gps_data_t *gpsdata)
 {
   char tmpbuf[255];
-  char maidenhead[5];
-  maidenhead[4]=0;
   int n;
-  char *s;
+  char *s, *gridsquare;
   int track;
 
   /* Get our location in Maidenhead. */
-  latlon2maidenhead(maidenhead,gpsdata->fix.latitude,gpsdata->fix.longitude);
+  gridsquare = maidenhead(gpsdata->fix.latitude,gpsdata->fix.longitude);
 
   /* Fill in the latitude and longitude. */
   if (gpsdata->fix.mode >= MODE_2D) {
@@ -255,7 +223,7 @@ static void update_lcd(struct gps_data_t *gpsdata)
     for(n=0;n<CLIMB;n++) avgclimb+=climb[n];
     avgclimb/=CLIMB;
     snprintf(tmpbuf, 254, "widget_set gpsd four 1 4 {%d %s %s %d fpm       }\n",
-            (int)(gpsdata->fix.altitude*altfactor), altunits, maidenhead, (int)(avgclimb * METERS_TO_FEET * 60));
+            (int)(gpsdata->fix.altitude*altfactor), altunits, gridsquare, (int)(avgclimb * METERS_TO_FEET * 60));
   } else {
     snprintf(tmpbuf, 254, "widget_set gpsd four 1 4 {n/a}\n");
   }
