@@ -143,6 +143,10 @@ void gpsd_init(struct gps_device_t *session, struct gps_context_t *context,
     session->gpsdata.epe = NAN;
     session->mag_var = NAN;
     session->gpsdata.dev.cycle = session->gpsdata.dev.mincycle = 1;
+#ifdef TIMING_ENABLE
+    session->sor = 0.0;
+    session->chars = 0;
+#endif /* TIMING_ENABLE */
 
     /* tty-level initialization */
     gpsd_tty_init(session);
@@ -306,14 +310,19 @@ int gpsd_activate(struct gps_device_t *session)
 
 	    /*@ -mustfreeonly @*/
 	    for (dp = gpsd_drivers; *dp; dp++) {
-		(void)tcflush(session->gpsdata.gps_fd, TCIOFLUSH);  /* toss stale data */
-		if ((*dp)->probe_detect != NULL
-			&& (*dp)->probe_detect(session) != 0) {
-		    gpsd_report(LOG_PROG, "probe found %s driver...\n",
-			    (*dp)->type_name);
-		    session->device_type = *dp;
-		    gpsd_assert_sync(session);
-		    goto foundit;
+		if ((*dp)->probe_detect != NULL) {
+		    gpsd_report(LOG_PROG, "Probing \"%s\" driver...\n",
+				 (*dp)->type_name);
+		    (void)tcflush(session->gpsdata.gps_fd, TCIOFLUSH);  /* toss stale data */
+		    if ((*dp)->probe_detect(session) != 0) {
+			gpsd_report(LOG_PROG, "Probe found \"%s\" driver...\n",
+				     (*dp)->type_name);
+			session->device_type = *dp;
+			gpsd_assert_sync(session);
+			goto foundit;
+		    } else
+			gpsd_report(LOG_PROG, "Probe not found \"%s\" driver...\n",
+			             (*dp)->type_name);
 		}
 	    }
 	    /*@ +mustfreeonly @*/
@@ -555,7 +564,7 @@ static gps_mask_t fill_dop(const struct gps_data_t * gpsdata, struct dop_t * dop
 
     /* If we don't have 4 satellites then we don't have enough information to calculate DOPS */
     if (n < 4) {
-	gpsd_report(LOG_DATA + 2, "Not enough Satellites available %d < 4:\n",
+	gpsd_report(LOG_DATA + 2, "Not enough satellites available %d < 4:\n",
 		    n);
 	return 0;		/* Is this correct return code here? or should it be ERROR_SET */
     }
@@ -811,8 +820,63 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
     gps_clear_fix(&session->newdata);
 
 #ifdef TIMING_ENABLE
+    /*
+     * Input just gecame available from a sensor, but no read from the
+     * device has yet been done.
+     *
+     * If we ever want a start-of-packet timestamp again, take it here
+     * with a test that session->packet.outbuflen is zero.
+     *
+     * What we actually do here is trickier.  For latency-timing
+     * purposes, we want to know the time at the start of the current
+     * recording cycle. We rely on the fact that even at 4800bps
+     * there's a quiet time perceptible to the human eye in gpsmon
+     * between when the last character of the last packet in a
+     * 1-second cycle ships and when the next reporting cycle
+     * ships. Because the cycle time is fixed, higher baud rates will
+     * make this gap larger.
+     *
+     * Thus, we look for an inter-character delay much larger than an
+     * average 4800bps sentence time.  How should this delay be set?  Well,
+     * counting framing bits and erring on the side of caution, it's 
+     * about 480 characters per second or 2083 microeconds per character;
+     * that's almost exactly 0.125 seconds per average 60-char sentence.
+     * Doubling this to avoid false positives, we look for an inter-character
+     * delay of greater than 0.250s.
+     *
+     * The above assumes a cycle time of 1 second.  To get the minimum size of 
+     * the quiet period, we multiply by the device cycle time.
+     *
+     * We can sanity-check these calculation by watching logs. If we have set 
+     * MINIMUM_QUIET_TIME correctly, the "transmission pause" message below
+     * will consistently be emitted just before the sentence that shows up
+     * as start-of-cycle in gpsmon, and never emitted at any other point
+     * in the cycle.
+     *
+     * In practice, it seems that edge detection succeeds at 9600bps but
+     * fails at 4800bps.  This is not surprsing, as previous profiling has 
+     * indicated that at 4800bps some devices overrun a 1-second cycle time 
+     * with the data they transmit.
+     */
+#define MINIMUM_QUIET_TIME	0.25
     if (session->packet.outbuflen == 0)
-	session->d_xmit_time = timestamp();
+    { 
+	/* beginning of a new packet */
+	timestamp_t now = timestamp();
+	if (session->device_type != NULL && session->packet.start_time > 0) {
+	    double quiet_time = (MINIMUM_QUIET_TIME * session->device_type->min_cycle);
+	    double gap = now - session->packet.start_time;
+
+	    if (gap > session->device_type->min_cycle)
+		gpsd_report(LOG_WARN, "cycle-start detector failed.\n");
+	    else if (gap > quiet_time) {
+		gpsd_report(LOG_PROG, "transmission pause of %f\n", gap);
+		session->sor = now;
+		session->packet.start_char = session->packet.char_counter;
+	    }
+	}
+	session->packet.start_time = now;
+    }
 #endif /* TIMING_ENABLE */
 
     if (session->packet.type >= COMMENT_PACKET) {
@@ -890,10 +954,6 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
 	gpsd_report(LOG_RAW + 3, "Accepted packet on %s.\n",
 		    session->gpsdata.dev.path);
 
-#ifdef TIMING_ENABLE
-	session->d_recv_time = timestamp();
-#endif /* TIMING_ENABLE */
-
 	/* track the packet count since achieving sync on the device */
 	if (first_sync) {
 	    speed_t speed = gpsd_get_speed(&session->ttyset);
@@ -947,6 +1007,14 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
 		&& session->device_type->parse_packet != NULL)
 		received |= session->device_type->parse_packet(session);
 
+#ifdef TIMING_ENABLE
+	/* are we going to generate a report? if so, count characters */
+	if ((received & REPORT_IS) != 0) {
+	    session->chars = session->packet.char_counter - session->packet.start_char;
+	}
+#endif /* TIMING_ENABLE */
+
+
 	session->gpsdata.set = ONLINE_SET | received;
 
 #ifdef CHEAPFLOATS_ENABLE
@@ -997,10 +1065,6 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
 	    session->fixcnt++;
 	}
 
-#ifdef TIMING_ENABLE
-	session->d_decode_time = timestamp();
-#endif /* TIMING_ENABLE */
-
 	/*
 	 * Sanity check.  This catches a surprising number of port and
 	 * driver errors, including 32-vs.-64-bit problems.
