@@ -63,6 +63,7 @@ static char *dgpsvec[] = {
 
 static bool sirf_initialize(void)
 {
+    /*@-globstate@*/
     unsigned int i;
 
     /*@ -onlytrans @*/
@@ -208,6 +209,7 @@ static bool sirf_initialize(void)
     gpsd_time_init(session.context, time(NULL));
 
     return true;
+    /*@+globstate@*/
 }
 
 static void decode_ecef(double x, double y, double z,
@@ -219,6 +221,9 @@ static void decode_ecef(double x, double y, double z,
     const double e_2 = (a * a - b * b) / (b * b);
     double lambda, p, theta, phi, n, h, vnorth, veast, vup, speed, heading;
 
+    /* splint pacification */
+    assert(mid2win!=NULL);
+
     lambda = atan2(y, x);
     /*@ -evalorder @*/
     p = sqrt(pow(x, 2) + pow(y, 2));
@@ -273,7 +278,9 @@ static void sirf_update(void)
     uint8_t dgps;
     char tbuf[JSON_DATE_MAX+1];
 
-    assert(mid27win != NULL);
+    /* splint pacification */
+    assert(mid2win!=NULL && mid27win != NULL);
+
     buf = session.packet.outbuffer + 4;
     len = session.packet.outbuflen - 8;
     switch (buf[0]) {
@@ -467,7 +474,8 @@ static void sirf_update(void)
 
     case 0x1b:
 	/******************************************************************
-	 Not actually documented in any published materials.
+	 Not actually documented in any published materials before the
+	 1.6 version of the SiRF binary protocol manual.
 	 Here is what Chris Kuethe got from the SiRF folks,
 	 (plus some corrections from the GpsPaSsion forums):
 
@@ -527,8 +535,8 @@ static void sirf_update(void)
 	monitor_log("NL  0x%02x=", buf[0]);
 	break;
 
-    case 0x29:			/* Geodetic Navigation Message */
-	monitor_log("GNM 0x29=");
+    case 0x29:			/* Geodetic Navigation Data */
+	monitor_log("GND 0x29=");
 	break;
 
     case 0x32:			/* SBAS Parameters */
@@ -541,71 +549,6 @@ static void sirf_update(void)
 	monitor_log("PPS 0x34=");
 	break;
 
-#ifdef __UNUSED__
-    case 0x62:
-	attrset(A_BOLD);
-	move(2, 40);
-	printw("%9.5f %9.5f", (double)(RAD_2_DEG * 1e8 * getbes32(buf, 1)),
-	       (double)(RAD_2_DEG * 1e8 * getbes32(buf, 5)));
-	move(2, 63);
-	printw("%8d", getbes32(buf, 9) / 1000);
-
-	move(3, 63);
-
-	printw("%8.1f", (double)getbes32(buf, 17) / 1000);
-
-	move(4, 54);
-	if (getbeu32(buf, 13) > 50) {
-	    double heading = RAD_2_DEG * 1e8 * getbes32(buf, 21);
-	    if (heading < 0)
-		heading += 360;
-	    printw("%5.1f", heading);
-	} else
-	    printw("  0.0");
-
-	move(4, 63);
-	printw("%8.1f", (double)getbes32(buf, 13) / 1000);
-	attrset(A_NORMAL);
-
-	move(5, 13);
-	printw("%04d-%02d-%02d %02d:%02d:%02d.%02d",
-	       (int)getbeu16(buf, 26), getub(buf, 28), getub(buf, 29),
-	       getub(buf, 30), getub(buf, 31), (unsigned short)getbeu16(buf,
-								       32) /
-	       1000, ((unsigned short)getbeu16(buf, 32) % 1000) / 10);
-	{
-	    struct timeval clk, gps;
-	    struct tm tm;
-
-	    gettimeofday(&clk, NULL);
-
-	    memset(&tm, 0, sizeof(tm));
-	    tm.tm_sec = (unsigned short)getbeu16(buf, 32) / 1000;
-	    tm.tm_min = (int)getub(buf, 31);
-	    tm.tm_hour = (int)getub(buf, 30);
-	    tm.tm_mday = (int)getub(buf, 29);
-	    tm.tm_mon = (int)getub(buf, 28) - 1;
-	    tm.tm_year = (int)getbeu16(buf, 26) - 1900;
-
-	    gps.tv_sec = mkgmtime(&tm);
-	    gps.tv_usec =
-		(((unsigned short)getbeu16(buf, 32) % 1000) / 10) * 10000;
-
-	    move(5, 2);
-	    printw("           ");
-	    move(5, 2);
-#if 1
-	    printw("%ld", (gps.tv_usec - clk.tv_usec) +
-		   ((gps.tv_sec - clk.tv_sec) % 3600) * 1000000);
-#else
-	    printw("%ld %ld %ld %ld", gps.tv_sec % 3600, gps.tv_usec,
-		   clk.tv_sec % 3600, clk.tv_usec);
-#endif
-	}
-	monitor_log("??? 0x62=");
-	break;
-#endif /* __UNUSED__ */
-
     case 0xff:			/* Development Data */
 	/*@ +ignoresigns @*/
 	while (len > 0 && buf[len - 1] == '\n')
