@@ -458,8 +458,14 @@ int gpsd_serial_open(struct gps_device_t *session)
 	 * Tip from Chris Kuethe: the FIDI chip used in the Trip-Nav
 	 * 200 (and possibly other USB GPSes) gets completely hosed
 	 * in the presence of flow control.  Thus, turn off CRTSCTS.
+	 *
+	 * This is not ideal.  Setting no parity here will mean extra
+	 * initialization time for some devices, like certain Trimble
+	 * boards, that want 7O2 or other non-8N1 settings. But starting
+	 * the hunt loop at 8N1 will minimize the average sync time 
+	 * over all devices.
 	 */
-	session->ttyset.c_cflag &= ~(PARENB | PARODD | CRTSCTS);
+	session->ttyset.c_cflag &= ~(PARENB | PARODD | CRTSCTS | CSTOPB);
 	session->ttyset.c_cflag |= CREAD | CLOCAL;
 	session->ttyset.c_iflag = session->ttyset.c_oflag =
 	    session->ttyset.c_lflag = (tcflag_t) 0;
@@ -469,6 +475,13 @@ int gpsd_serial_open(struct gps_device_t *session)
 #endif /* FIXED_PORT_SPEED */
 	gpsd_set_speed(session, gpsd_get_speed(&session->ttyset_old), 'N', 1);
     }
+
+    /* required so parity field won't be '\0' if saved speed matches */
+    if (session->sourcetype <= source_blockdev) {
+	session->gpsdata.dev.parity = 'N';
+	session->gpsdata.dev.stopbits = 1;
+    }
+
     gpsd_report(LOG_SPIN, "open(%s) -> %d in gpsd_serial_open()\n",
 		session->gpsdata.dev.path, session->gpsdata.gps_fd);
     return session->gpsdata.gps_fd;
@@ -575,14 +588,22 @@ void gpsd_close(struct gps_device_t *session)
 	/* this is the clean way to do it */
 	session->ttyset_old.c_cflag |= HUPCL;
 	/* keep the most recent baud rate */
-	/*@ ignore @*/
-	(void)cfsetispeed(&session->ttyset_old,
-			  (speed_t) session->gpsdata.dev.baudrate);
-	(void)cfsetospeed(&session->ttyset_old,
-			  (speed_t) session->gpsdata.dev.baudrate);
-	/*@ end @*/
-	(void)tcsetattr(session->gpsdata.gps_fd, TCSANOW,
-			&session->ttyset_old);
+	/*
+	 * Don't revert the serial parameters if we didn't have to mess with
+	 * them the first time.  Economical, and avoids tripping over an
+	 * obscure Linux 2.6 kernel bug that disables threaded
+	 * ioctl(TIOCMWAIT) on a device after tcsetattr() is called.
+	 */
+	if (session->ttyset_old.c_ispeed != session->ttyset.c_ispeed || (session->ttyset_old.c_cflag & CSTOPB) != (session->ttyset.c_cflag & CSTOPB)) {
+	    /*@ ignore @*/
+	    (void)cfsetispeed(&session->ttyset_old,
+			      (speed_t) session->gpsdata.dev.baudrate);
+	    (void)cfsetospeed(&session->ttyset_old,
+			      (speed_t) session->gpsdata.dev.baudrate);
+	    /*@ end @*/
+	    (void)tcsetattr(session->gpsdata.gps_fd, TCSANOW,
+			    &session->ttyset_old);
+	}
 	gpsd_report(LOG_SPIN, "close(%d) in gpsd_close(%s)\n",
 		    session->gpsdata.gps_fd, session->gpsdata.dev.path);
 	(void)close(session->gpsdata.gps_fd);
