@@ -13,6 +13,12 @@
 
 using namespace std;
 
+/* 
+ * We should get libgps_dump_state() from the client library, but
+ * scons has a bug; we can't get it to add -lgps to the link line,
+ * apparently because it doesn't honor parse_flags on a Program()
+ * build of a C++ file.
+ */
 static void libgps_dump_state(struct gps_data_t *collect)
 {
     /* no need to dump the entire state, this is a sanity check */
@@ -98,7 +104,10 @@ int main(void)
 {
     gpsmm gps_rec("localhost", DEFAULT_GPSD_PORT);
 
-    gps_rec.stream(WATCH_ENABLE|WATCH_JSON);
+    if (gps_rec.stream(WATCH_ENABLE|WATCH_JSON) == NULL) {
+        cerr << "No GPSD running.\n";
+        return 1;
+    }
 
     for (;;) {
 	struct gps_data_t* newdata;
